@@ -15,10 +15,14 @@
 #  specific language governing permissions and limitations
 #  under the License.
 
+
+# ------- Read RDD ------------
+
 #' Create a SpatialRDD from an external data source.
+#'
 #' Import spatial object from an external data source into a Sedona SpatialRDD.
 #'
-#' @param sc A \code{spark_connection}.
+#' @param sc A `spark_connection`.
 #' @param location Location of the data source.
 #' @param type Type of the SpatialRDD (must be one of "point", "polygon", or
 #'   "linestring".
@@ -29,6 +33,7 @@
 #'   RDD (default: 1).
 #'
 #' @name sedona_spatial_rdd_data_source
+#' @keywords internal
 NULL
 
 #' Create a typed SpatialRDD from a delimiter-separated values data source.
@@ -38,11 +43,11 @@ NULL
 #' The data source can contain spatial attributes (e.g., longitude and latidude)
 #' and other attributes. Currently only inputs with spatial attributes occupying
 #' a contiguous range of columns (i.e.,
-#' [first_spatial_col_index, last_spatial_col_index]) are supported.
+#' \[first_spatial_col_index, last_spatial_col_index\]) are supported.
 #'
 #' @inheritParams sedona_spatial_rdd_data_source
 #' @param delimiter Delimiter within each record. Must be one of
-#'   ',', '\\t', '?', '\\'', '"', '_', '-', '\%', '~', '|', ';'
+#'   ',', '\\t', '?', '\\'', '"', '_', '-', '%', '~', '|', ';'
 #' @param first_spatial_col_index Zero-based index of the left-most column
 #'   containing spatial attributes (default: 0).
 #' @param last_spatial_col_index Zero-based index of the right-most column
@@ -71,7 +76,7 @@ NULL
 #'   )
 #' }
 #'
-#' @family Sedona data inferface functions
+#' @family Sedona RDD data interface functions
 #'
 #' @export
 sedona_read_dsv_to_typed_rdd <- function(sc,
@@ -146,12 +151,28 @@ sedona_read_dsv_to_typed_rdd <- function(sc,
     new_spatial_rdd(type)
 }
 
-#' Create a typed SpatialRDD from a shapefile data source.
+#' (Deprecated) Create a typed SpatialRDD from a shapefile or geojson data source.
+#'
+#' @description
+#' `r lifecycle::badge("deprecated")`
+#'
+#' Constructors of typed RDD (PointRDD, PolygonRDD, LineStringRDD) are soft deprecated, use non-types versions
 #'
 #' Create a typed SpatialRDD (namely, a PointRDD, a PolygonRDD, or a
-#' LineStringRDD) from a shapefile data source.
+#' LineStringRDD)
+#' * `sedona_read_shapefile_to_typed_rdd`: from a shapefile data source
+#' * `sedona_read_geojson_to_typed_rdd`: from a GeoJSON data source
 #'
-#' @inheritParams sedona_spatial_rdd_data_source
+#'
+#' @param sc A `spark_connection`.
+#' @param location Location of the data source.
+#' @param type Type of the SpatialRDD (must be one of "point", "polygon", or
+#'   "linestring".
+#' @param has_non_spatial_attrs Whether the input contains non-spatial
+#'   attributes.
+#' @param storage_level Storage level of the RDD (default: MEMORY_ONLY).
+#' @param repartition The minimum number of partitions to have in the resulting
+#'   RDD (default: 1).
 #'
 #' @return A typed SpatialRDD.
 #'
@@ -169,13 +190,20 @@ sedona_read_dsv_to_typed_rdd <- function(sc,
 #'   )
 #' }
 #'
-#' @family Sedona data inferface functions
+#' @family Sedona RDD data interface functions
 #'
 #' @export
 sedona_read_shapefile_to_typed_rdd <- function(sc,
                                                location,
                                                type = c("point", "polygon", "linestring"),
                                                storage_level = "MEMORY_ONLY") {
+
+  lifecycle::deprecate_soft(
+    "1.4.0",
+    "sedona_read_shapefile_to_typed_rdd()",
+    with = "sedona_read_shapefile()"
+  )
+
   invoke_static(
     sc,
     "org.apache.sedona.core.formatMapper.shapefileParser.ShapefileReader",
@@ -187,38 +215,22 @@ sedona_read_shapefile_to_typed_rdd <- function(sc,
     new_spatial_rdd(type)
 }
 
-#' Create a typed SpatialRDD from a GeoJSON data source.
-#'
-#' Create a typed SpatialRDD (namely, a PointRDD, a PolygonRDD, or a
-#' LineStringRDD) from a GeoJSON data source.
-#'
-#' @inheritParams sedona_spatial_rdd_data_source
-#'
-#' @return A typed SpatialRDD.
-#'
-#' @examples
-#' library(sparklyr)
-#' library(apache.sedona)
-#'
-#' sc <- spark_connect(master = "spark://HOST:PORT")
-#'
-#' if (!inherits(sc, "test_connection")) {
-#'   input_location <- "/dev/null" # replace it with the path to your input file
-#'   rdd <- sedona_read_geojson_to_typed_rdd(
-#'     sc,
-#'     location = input_location, type = "polygon"
-#'   )
-#' }
-#'
-#' @family Sedona data inferface functions
-#'
+
 #' @export
+#' @rdname sedona_read_shapefile_to_typed_rdd
 sedona_read_geojson_to_typed_rdd <- function(sc,
                                              location,
                                              type = c("point", "polygon", "linestring"),
                                              has_non_spatial_attrs = TRUE,
                                              storage_level = "MEMORY_ONLY",
                                              repartition = 1L) {
+
+  lifecycle::deprecate_soft(
+    "1.4.0",
+    "sedona_read_geojson_to_typed_rdd()",
+    with = "sedona_read_geojson()"
+  )
+
   invoke_new(
     sc,
     rdd_cls_from_type(type),
@@ -232,16 +244,30 @@ sedona_read_geojson_to_typed_rdd <- function(sc,
     new_spatial_rdd(type)
 }
 
-#' Create a SpatialRDD from a GeoJSON data source.
+
+
+#' Read geospatial data into a Spatial RDD
 #'
-#' Create a generic SpatialRDD from a GeoJSON data source.
+#' @description Import spatial object from an external data source into a Sedona SpatialRDD.
+#' * `sedona_read_shapefile`: from a shapefile
+#' * `sedona_read_geojson`: from a geojson file
+#' * `sedona_read_wkt`: from a geojson file
+#' * `sedona_read_wkb`: from a geojson file
 #'
-#' @inheritParams sedona_spatial_rdd_data_source
+#' @param sc A `spark_connection`.
+#' @param location Location of the data source.
+#' @param storage_level Storage level of the RDD (default: MEMORY_ONLY).
+#' @param repartition The minimum number of partitions to have in the resulting
+#'   RDD (default: 1).
 #' @param allow_invalid_geometries Whether to allow topology-invalid
 #'   geometries to exist in the resulting RDD.
 #' @param skip_syntactically_invalid_geometries Whether to allows Sedona to
 #'   automatically skip syntax-invalid geometries, rather than throwing
 #'   errorings.
+#' @param wkt_col_idx Zero-based index of column containing hex-encoded WKB data
+#'   (default: 0).
+#' @param wkb_col_idx Zero-based index of column containing hex-encoded WKB data
+#'   (default: 0).
 #'
 #' @return A SpatialRDD.
 #'
@@ -256,7 +282,7 @@ sedona_read_geojson_to_typed_rdd <- function(sc,
 #'   rdd <- sedona_read_geojson(sc, location = input_location)
 #' }
 #'
-#' @family Sedona data inferface functions
+#' @family Sedona RDD data interface functions
 #'
 #' @export
 sedona_read_geojson <- function(sc,
@@ -283,39 +309,9 @@ sedona_read_geojson <- function(sc,
     new_spatial_rdd(NULL)
 }
 
-#' Create a SpatialRDD from a Well-Known Binary (WKB) data source.
-#'
-#' Create a generic SpatialRDD from a hex-encoded Well-Known Binary (WKB) data
-#' source.
-#'
-#' @inheritParams sedona_spatial_rdd_data_source
-#' @param wkb_col_idx Zero-based index of column containing hex-encoded WKB data
-#'   (default: 0).
-#' @param allow_invalid_geometries Whether to allow topology-invalid
-#'   geometries to exist in the resulting RDD.
-#' @param skip_syntactically_invalid_geometries Whether to allows Sedona to
-#'   automatically skip syntax-invalid geometries, rather than throwing
-#'   errorings.
-#'
-#' @return A SpatialRDD.
-#'
-#' @examples
-#' library(sparklyr)
-#' library(apache.sedona)
-#'
-#' sc <- spark_connect(master = "spark://HOST:PORT")
-#'
-#' if (!inherits(sc, "test_connection")) {
-#'   input_location <- "/dev/null" # replace it with the path to your input file
-#'   rdd <- sedona_read_wkb(
-#'     sc,
-#'     location = input_location, wkb_col_idx = 0L
-#'   )
-#' }
-#'
-#' @family Sedona data inferface functions
-#'
+
 #' @export
+#' @rdname sedona_read_geojson
 sedona_read_wkb <- function(sc,
                             location,
                             wkb_col_idx = 0L,
@@ -343,39 +339,9 @@ sedona_read_wkb <- function(sc,
     new_spatial_rdd(NULL)
 }
 
-#' Create a SpatialRDD from a Well-Known Text (WKT) data source.
-#'
-#' Create a generic SpatialRDD from a Well-Known Text (WKT) data source.
-#'
-#' @inheritParams sedona_spatial_rdd_data_source
-#' @param wkt_col_idx Zero-based index of column containing hex-encoded WKB data
-#'   (default: 0).
-#' @param allow_invalid_geometries Whether to allow topology-invalid
-#'   geometries to exist in the resulting RDD.
-#' @param skip_syntactically_invalid_geometries Whether to allows Sedona to
-#'   automatically skip syntax-invalid geometries, rather than throwing
-#'   errorings.
-#'
-#' @return A SpatialRDD.
-#'
-#' @examples
-#' library(sparklyr)
-#' library(apache.sedona)
-#'
-#' sc <- spark_connect("spark://HOST:PORT")
-#'
-#' if (!inherits(sc, "test_connection")) {
-#'   input_location <- "/dev/null" # replace it with the path to your input file
-#'   rdd <- sedona_read_wkt(
-#'     sc,
-#'     location = input_location,
-#'     wkt_col_idx = 0L
-#'   )
-#' }
-#'
-#' @family Sedona data inferface functions
-#'
+
 #' @export
+#' @rdname sedona_read_geojson
 sedona_read_wkt <- function(sc,
                             location,
                             wkt_col_idx = 0L,
@@ -403,27 +369,7 @@ sedona_read_wkt <- function(sc,
     new_spatial_rdd(NULL)
 }
 
-#' Create a SpatialRDD from a shapefile data source.
-#'
-#' Create a generic SpatialRDD from a shapefile data source.
-#'
-#' @inheritParams sedona_spatial_rdd_data_source
-#'
-#' @return A SpatialRDD.
-#'
-#' @examples
-#' library(sparklyr)
-#' library(apache.sedona)
-#'
-#' sc <- spark_connect(master = "spark://HOST:PORT")
-#'
-#' if (!inherits(sc, "test_connection")) {
-#'   input_location <- "/dev/null" # replace it with the path to your input file
-#'   rdd <- sedona_read_shapefile(sc, location = input_location)
-#' }
-#'
-#' @family Sedona data inferface functions
-#'
+#' @rdname sedona_read_geojson
 #' @export
 sedona_read_shapefile <- function(sc,
                                   location,
@@ -439,22 +385,126 @@ sedona_read_shapefile <- function(sc,
     new_spatial_rdd(NULL)
 }
 
+
+# ------- Read SDF ------------
+#' Read geospatial data into a Spark DataFrame.
+#'
+#' @description Functions to read geospatial data from a variety of formats into Spark DataFrames.
+#'
+#' * `spark_read_shapefile`: from a shapefile
+#' * `spark_read_geojson`: from a geojson file
+#' * `spark_read_geoparquet`: from a geoparquet file
+#'
+#' @inheritParams sparklyr::spark_read_source
+#'
+#'
+#' @return A tbl
+#'
+#' @examples
+#' library(sparklyr)
+#' library(apache.sedona)
+#'
+#' sc <- spark_connect(master = "spark://HOST:PORT")
+#'
+#' if (!inherits(sc, "test_connection")) {
+#'   input_location <- "/dev/null" # replace it with the path to your input file
+#'   rdd <- spark_read_shapefile(sc, location = input_location)
+#' }
+#'
+#' @family Sedona DF data interface functions
+#'
+#' @export
+spark_read_shapefile <- function(sc,
+                                 name = NULL,
+                                 path = name,
+                                 options = list(),
+                                 ...) {
+
+  lapply(names(options), function(name) {
+    if (!name %in% c("")) {
+      warning(paste0("Ignoring unknown option '", name,"'"))
+    }
+  })
+
+  rdd <- sedona_read_shapefile(sc,
+                               location = path,
+                               storage_level = "MEMORY_ONLY")
+
+
+
+  rdd %>% sdf_register(name = name)
+}
+
+
+#' @export
+#' @rdname spark_read_shapefile
+spark_read_geojson <- function(sc,
+                               name = NULL,
+                               path = name,
+                               options = list(),
+                               repartition = 0,
+                               memory = TRUE,
+                               overwrite = TRUE) {
+
+  # check options
+  if ("allow_invalid_geometries" %in% names(options)) final_allow_invalid <- options[["allow_invalid_geometries"]] else final_allow_invalid <- TRUE
+  if ("skip_syntactically_invalid_geometries" %in% names(options)) final_skip <- options[["skip_syntactically_invalid_geometries"]] else final_skip <- TRUE
+  lapply(names(options), function(name) {
+    if (!name %in% c("allow_invalid_geometries", "skip_syntactically_invalid_geometries")) {
+      warning(paste0("Ignoring unknown option '", name,"'"))
+    }
+  })
+
+  final_repartition <- max(as.integer(repartition), 1L)
+
+  rdd <- sedona_read_geojson(sc,
+                             location = path,
+                             allow_invalid_geometries = final_allow_invalid,
+                             skip_syntactically_invalid_geometries = final_skip,
+                             storage_level = "MEMORY_ONLY",
+                             repartition = final_repartition)
+
+
+
+  rdd %>% sdf_register(name = name)
+}
+
+#' @export
+#' @rdname spark_read_shapefile
+#' @importFrom sparklyr spark_read_source
+spark_read_geoparquet <- function(sc,
+                                  name = NULL,
+                                  path = name,
+                                  options = list(),
+                                  repartition = 0,
+                                  memory = TRUE,
+                                  overwrite = TRUE) {
+
+  spark_read_source(sc,
+                    name = name,
+                    path = path,
+                    source = "geoparquet",
+                    options = options,
+                    repartition = repartition,
+                    memory = memory,
+                    overwrite = overwrite,
+                    columns = NULL)
+}
+
+
+# ------- Write RDD ------------
+
+
 #' Write SpatialRDD into a file.
 #'
-#' Export serialized data from a Sedona SpatialRDD into an output file.
+#' Export serialized data from a Sedona SpatialRDD into a file.
+#' * `sedona_write_wkb`:
+#' * `sedona_write_wkt`:
+#' * `sedona_write_geojson`:
 #'
 #' @param x The SpatialRDD object.
 #' @param output_location Location of the output file.
 #'
-#' @name sedona_spatial_rdd_serialization_routine
-NULL
-
-#' Write SpatialRDD into a WKB file.
-#'
-#' Export serialized data from a Sedona SpatialRDD into a WKB file.
-#'
-#' @inheritParams sedona_spatial_rdd_serialization_routine
-#'
 #' @return No return value.
 #'
 #' @examples
@@ -473,70 +523,23 @@ NULL
 #'   sedona_write_wkb(rdd, "/tmp/wkb_output.tsv")
 #' }
 #'
-#' @family Sedona data inferface functions
+#' @family Sedona RDD data interface functions
 #'
 #' @export
 sedona_write_wkb <- function(x, output_location) {
   invoke(x$.jobj, "saveAsWKB", output_location)
 }
 
-#' Write SpatialRDD into a WKT file.
-#'
-#' Export serialized data from a Sedona SpatialRDD into a WKT file.
-#'
-#' @inheritParams sedona_spatial_rdd_serialization_routine
-#'
-#' @return No return value.
-#'
-#' @examples
-#' library(sparklyr)
-#' library(apache.sedona)
-#'
-#' sc <- spark_connect("spark://HOST:PORT")
-#'
-#' if (!inherits(sc, "test_connection")) {
-#'   input_location <- "/dev/null" # replace it with the path to your input file
-#'   rdd <- sedona_read_wkt(
-#'     sc,
-#'     location = input_location,
-#'     wkt_col_idx = 0L
-#'   )
-#'   sedona_write_wkt(rdd, "/tmp/wkt_output.tsv")
-#' }
-#'
-#' @family Sedona data inferface functions
-#'
+
 #' @export
+#' @rdname sedona_write_wkb
 sedona_write_wkt <- function(x, output_location) {
   invoke(x$.jobj, "saveAsWKT", output_location)
 }
 
-#' Write SpatialRDD into a GeoJSON file.
-#'
-#' Export serialized data from a Sedona SpatialRDD into a GeoJSON file.
-#'
-#' @inheritParams sedona_spatial_rdd_serialization_routine
-#'
-#' @return No return value.
-#'
-#' @examples
-#' library(sparklyr)
-#' library(apache.sedona)
-#'
-#' sc <- spark_connect(master = "spark://HOST:PORT")
-#'
-#' if (!inherits(sc, "test_connection")) {
-#'   input_location <- "/dev/null" # replace it with the path to your input file
-#'   rdd <- sedona_read_geojson_to_typed_rdd(
-#'     sc,
-#'     location = input_location, type = "polygon"
-#'   )
-#'   sedona_write_geojson(rdd, "/tmp/example.json")
-#' }
-#'
-#' @family Sedona data inferface functions
-#'
+
 #' @export
+#' @rdname sedona_write_wkb
 sedona_write_geojson <- function(x, output_location) {
   invoke(x$.jobj, "saveAsGeoJSON", output_location)
 }
@@ -573,7 +576,7 @@ sedona_write_geojson <- function(x, output_location) {
 #'   )
 #' }
 #'
-#' @family Sedona data inferface functions
+#' @family Sedona RDD data interface functions
 #'
 #' @export
 sedona_save_spatial_rdd <- function(x,
@@ -589,6 +592,131 @@ sedona_save_spatial_rdd <- function(x,
   )
 }
 
+
+# ------- Write SDF ------------
+
+
+### No shapefile writer in Sedona
+
+
+
+#' Write geospatial data from a Spark DataFrame.
+#'
+#' @description Functions to write geospatial data into a variety of formats from Spark DataFrames.
+#'
+#' * `spark_write_geojson`: to GeoJSON
+#' * `spark_write_geoparquet`: to GeoParquet
+#' * `spark_write_raster`: to raster tiles after using RS output functions (`RS_AsXXX`)
+#'
+#'
+#' @param path The path to the file. Needs to be accessible from the cluster.
+#'   Supports the \samp{"hdfs://"}, \samp{"s3a://"} and \samp{"file://"} protocols.
+#' @inheritParams sparklyr::spark_write_source
+#'
+#'
+#' @return NULL
+#'
+#' @examples
+#' library(sparklyr)
+#' library(apache.sedona)
+#'
+#' sc <- spark_connect(master = "spark://HOST:PORT")
+#'
+#' if (!inherits(sc, "test_connection")) {
+#'   tbl <- dplyr::tbl(
+#'     sc,
+#'     dplyr::sql("SELECT ST_GeomFromText('POINT(-71.064544 42.28787)') AS `pt`")
+#'   )
+#'   spark_write_geojson(
+#'     tbl %>% dplyr::mutate(id = 1),
+#'     output_location = "/tmp/pts.geojson"
+#'   )
+#' }
+#'
+#' @family Sedona DF data interface functions
+#'
+#' @importFrom sparklyr spark_write_source
+#' @export
+spark_write_geojson <- function(x,
+                                path,
+                                mode = NULL,
+                                options = list(),
+                                partition_by = NULL,
+                                ...) {
+
+  ## find geometry column if not specified
+  if (!"spatial_col" %in% names(options)) {
+    schema <- x %>% sdf_schema()
+    potential_cols <- which(sapply(schema, function(x) x$type == "GeometryUDT"))
+
+    if (length(potential_cols) == 0) {
+      cli::cli_abort("No geometry column found")
+    } else if (length(potential_cols) > 1) {
+      spatial_col = names(potential_cols)[1]
+      cli::cli_warn("Multiple geometry columns found, using {spatial_col}")
+    } else {
+      spatial_col = names(potential_cols)
+    }
+
+  } else {
+    spatial_col = options[["spatial_col"]]
+  }
+
+  rdd <- x %>% to_spatial_rdd(spatial_col = spatial_col)
+
+  sedona_write_geojson(x = rdd, output_location = path)
+
+}
+
+
+#' @export
+#' @rdname spark_write_geojson
+#' @importFrom sparklyr spark_write_source
+spark_write_geoparquet <- function(x,
+                                   path,
+                                   mode = NULL,
+                                   options = list(),
+                                   partition_by = NULL,
+                                   ...) {
+
+  spark_write_source(
+    x = x,
+    source = "geoparquet",
+    mode = mode,
+    options = options,
+    partition_by = partition_by,
+    save_args = list(path),
+    ...
+  )
+
+}
+
+
+#' @export
+#' @rdname spark_write_geojson
+#' @importFrom sparklyr spark_write_source
+spark_write_raster <- function(x,
+                                   path,
+                                   mode = NULL,
+                                   options = list(),
+                                   partition_by = NULL,
+                                   ...) {
+
+  spark_write_source(
+    x = x,
+    source = "raster",
+    mode = mode,
+    options = options,
+    partition_by = partition_by,
+    save_args = list(path),
+    ...
+  )
+
+}
+
+
+
+# ------- Utilities ------------
 rdd_cls_from_type <- function(type = c("point", "polygon", "linestring")) {
   type <- match.arg(type)
 
